@@ -19,6 +19,12 @@ class CubeBackend(abc.ABC):
         """
         return list(self.get_records(query))[0]
 
+    @abc.abstractmethod
+    def get_count(self, query: CubeQuery) -> int:
+        """
+        Returns the number of records returned by a query
+        """
+
     @abc.abstractmethod
     def store_records(self, cube: Type[Cube], records: Iterable[NamedTuple]):
         """
