@@ -71,7 +71,7 @@ class Cube(metaclass=CubeMeta):
         try:
             return cls._dimensions[name]
         except KeyError:
-            raise ConfigurationError("Unknown dimension: {name}")
+            raise ConfigurationError(f"Unknown dimension: {name}")
 
     @classmethod
     def metric_by_name(cls, name: str) -> Metric:
