@@ -30,6 +30,7 @@ from hcube.api.models.materialized_views import AggregatingMaterializedView
 from hcube.api.models.metrics import FloatMetric, Metric, IntMetric
 from hcube.api.models.query import CubeQuery
 from hcube.api.models.transforms import Transform, ExplicitMappingTransform, StoredMappingTransform
+from hcube.settings import GlobalSettings
 
 logger = logging.getLogger(__name__)
 
@@ -94,10 +95,7 @@ class ClickhouseCubeBackend(CubeBackend):
         (FloatMetric, "Float"),
         (IntMetric, "Int"),
     )
-    default_settings = {
-        "aggregate_functions_null_for_empty": 1,
-        "allow_experimental_projection_optimization": 1,
-    }
+    default_settings = {}
 
     def __init__(
         self,
@@ -108,9 +106,7 @@ class ClickhouseCubeBackend(CubeBackend):
         super().__init__()
         self.database = database
         assert self.database, "database must be present"
-        self.query_settings = {**self.default_settings}
-        if query_settings:
-            self.query_settings.update(query_settings)
+        self.query_settings = query_settings or {}
         self.client_attrs = client_attrs
         self.pool = ChPool(database=database, **client_attrs)
         self._table_exists = {}
@@ -162,6 +158,16 @@ class ClickhouseCubeBackend(CubeBackend):
             for rec in output:
                 yield result(*rec)
 
+    def get_count(self, query: CubeQuery) -> int:
+        text, params, *_ = self._prepare_db_query(query)
+        text = f"SELECT COUNT() FROM ({text}) AS _count"
+        logger.debug('Query: "%s", params: "%s"', text, params)
+        start = monotonic()
+        with self.pool.get_client() as client:
+            output = client.execute(text, params)
+            logger.debug(f"Query time: {monotonic() - start: .3f} s")
+            return output[0][0]
+
     def delete_records(self, query: CubeQuery) -> None:
         """
         In clickhouse we do not delete the records, but rather insert the same records with
@@ -175,20 +181,85 @@ class ClickhouseCubeBackend(CubeBackend):
             raise ConfigurationError(
                 "Delete query can only have a filter, no aggregations, group_bys or transforms"
             )
-        text, params, fields, _matview = self._prepare_db_query(
-            query, auto_use_materialized_views=False, append_to_select=", -1"
-        )
-        table = f"{self.database}.{self.cube_to_table_name(query.cube)}"
-        field_names = ",".join(fields)
-        meta = self.get_table_meta(query.cube)
 
-        whole_text = f"INSERT INTO {table} ({field_names}, {meta.sign_col}) {text}"
+        meta = self.get_table_meta(query.cube)
+        table = f"{self.database}.{self.cube_to_table_name(query.cube)}"
+        where_parts = []
+        params = {}
+        for fltr in query.filters:
+            filter_text, filter_params = self._ch_filter(fltr)
+            where_parts.append(filter_text)
+            params.update(filter_params)
+        where = " AND ".join(where_parts)
+
+        dims = [dim.name for dim in query.cube._dimensions.values()]
+        metrics = [metric.name for metric in query.cube._metrics.values()]
+        dim_names = ",".join(dims)
+        metric_names = ",".join(metrics)
+        metric_sums = ",".join(f"sum({metric}*{meta.sign_col}) as {metric}" for metric in metrics)
+
+        # In a previous version, we simply inserted the records returned by a select with FINAL
+        # keyword with the opposite sign. But it turns out that the FINAL keyword has some problems
+        # - in newer versions of clickhouse a special setting has to be given to use skip indexes
+        # with FINAL and it still has strange performance issues - it seems that skip indexes are
+        # not used for FINAL queries by default, this code path is not well tested.
+        #
+        # This is why we use a different approach which gets around the FINAL keyword by using
+        # an aggregation query to get the sums of the metrics and then insert these values with
+        # the opposite sign.
+        #
+        # Please note that we could not just insert plain records with opposite sign without using
+        # final, because if there already were records with the -1 sign, we could just duplicate
+        # both the positive and negative records. By using the aggregation query, we are sure that
+        # only the positive records are negated by inserting records with the opposite sign.
+
+        whole_text = (
+            f"INSERT INTO {table} ({dim_names}, {metric_names}, {meta.sign_col}) "
+            f"SELECT {dim_names}, {metric_sums}, -1 "
+            f"FROM {table} "
+            f"WHERE {where} "
+            f"GROUP BY {dim_names} "
+            f"HAVING SUM({meta.sign_col}) > 0"
+        )
         logger.debug("Delete query: %s, params: %s", whole_text, params)
         start = monotonic()
         with self.pool.get_client() as client:
             client.execute(whole_text, params)
             logger.debug(f"Query time: {monotonic() - start: .3f} s")
 
+    def delete_records_hard(self, query: CubeQuery) -> None:
+        """
+        Clickhouse has a DELETE command, but it is not very efficient. We support it by this extra
+        method, but it is not recommended to use it.
+        """
+        logger.warning("Hard delete is not recommended for Clickhouse, it performs poorly")
+        self.create_table(query.cube)  # ensure the table exists
+        # check that the query can be used
+        if query.aggregations or query.groups or query.transforms:
+            raise ConfigurationError(
+                "Delete query can only have a filter, no aggregations, group_bys or transforms"
+            )
+
+        table = f"{self.database}.{self.cube_to_table_name(query.cube)}"
+        where_parts = []
+        params = {}
+        for fltr in query.filters:
+            filter_text, filter_params = self._ch_filter(fltr)
+            where_parts.append(filter_text)
+            params.update(filter_params)
+        where = " AND ".join(where_parts)
+
+        # put it together
+        text = f"ALTER TABLE {table} DELETE "
+        if where:
+            text += f"WHERE {where} "
+
+        logger.debug("Delete query: %s, params: %s", text, params)
+        start = monotonic()
+        with self.pool.get_client() as client:
+            client.execute(text, params)
+            logger.debug(f"Query time: {monotonic() - start: .3f} s")
+
     @classmethod
     def get_table_meta(cls, cube: Type[Cube]) -> TableMetaParams:
         meta = TableMetaParams()
@@ -263,10 +334,8 @@ class ClickhouseCubeBackend(CubeBackend):
             text += f"ORDER BY {order_by} "
         if query.limit:
             text += f"LIMIT {query.limit} "
-        # ensure MAX([]) is None not 0
-        # but this does not seem to be compatible with automatic use of projections, so we
-        # might need to turn it off in the future
-        applied_settings = {**self.default_settings}
+        # get suitable settings for the query
+        applied_settings = self._get_query_settings()
         if final and meta.use_skip_indexes_if_final:
             applied_settings["use_skip_indexes_if_final"] = 1
         settings_part = ", ".join(f"{k} = {v}" for k, v in applied_settings.items())
@@ -274,6 +343,12 @@ class ClickhouseCubeBackend(CubeBackend):
             text += f" SETTINGS {settings_part}"
         return text, params, fields, matview
 
+    def _get_query_settings(self) -> dict:
+        out = {**self.default_settings, **self.query_settings}
+        if not GlobalSettings.aggregates_zero_for_empty_data:
+            out["aggregate_functions_null_for_empty"] = 1
+        return out
+
     def _translate_aggregation(self, agg: Aggregation, sign_column: Optional[str]) -> str:
         agg_name = self._agg_name(agg.op)
         inside = ""
